@@ -108,10 +108,8 @@ def maxcut_hamming_cdf(n_qubits, J_func, degrees, quality, hamming_prob):
 
 # Written by Elara (OpenAI custom GPT) and improved by Dan Strano
 @njit
-def local_repulsion_choice(adjacency_data, adjacency_rows, weights, n, m):
+def local_repulsion_choice(G_m, max_weight, weights, n, m):
     """
-
-
     Pick m nodes out of n with repulsion bias:
     - High-degree nodes are already less likely
     - After choosing a node, its neighbors' probabilities are further reduced
@@ -155,10 +153,9 @@ def local_repulsion_choice(adjacency_data, adjacency_rows, weights, n, m):
         mask[node] = True
 
         # Repulsion: penalize neighbors
-        for j in range(adjacency_rows[node], adjacency_rows[node + 1]):
-            nbr = adjacency_data[j]
+        for nbr in range(n):
             if available[nbr]:
-                weights[nbr] *= 0.5  # tunable penalty factor
+                weights[nbr] *= 0.5 ** (G_m[node, nbr] / max_weight)  # tunable penalty factor
 
     return mask
 
@@ -173,29 +170,9 @@ def compute_energy(sample, G_m, n_qubits):
     return energy
 
 
-@njit(parallel=True)
-def compute_adjacency(G_m, size):
-    n_qubits = G_m.shape[0]
-    adjacency_rows = np.empty(n_qubits + 1, dtype=np.int32)
-    adjacency_rows[0] = 0
-    adjacency_data = np.full(size, -1, dtype=np.int32)
-    k = 0
-    for i in prange(n_qubits):
-        for j in range(n_qubits):
-            if (i == j) or (G_m[i, j] <= 0.0):
-                continue
-            adjacency_data[k] = j
-            k += 1
-        adjacency_rows[i + 1] = k
-
-    np.resize(adjacency_data, k)
-
-    return adjacency_data, adjacency_rows
-
-
 @njit(parallel=True)
 def sample_for_solution(G_m, shots, thresholds, degrees_sum, J_eff, n):
-    adjacency_data, adjacency_rows = compute_adjacency(G_m, degrees_sum)
+    max_weight = G_m.max()
     weights = 1.0 / (1.0 + (2 ** -52) - J_eff)
 
     solutions = np.empty((shots, n), dtype=np.bool_)
@@ -210,7 +187,7 @@ def sample_for_solution(G_m, shots, thresholds, degrees_sum, J_eff, n):
         m += 1
 
         # Second dimension: permutation within Hamming weight
-        sample = local_repulsion_choice(adjacency_data, adjacency_rows, weights, n, m)
+        sample = local_repulsion_choice(G_m, max_weight, weights, n, m)
 
         solutions[s] = sample
         energies[s] = compute_energy(sample, G_m, n)
@@ -302,25 +279,26 @@ def maxcut_tfim(
         nodes = list(range(n_qubits))
         G_m = G
 
-    if n_qubits == 0:
-        return "", 0, ([], [])
+    if n_qubits < 3:
+        if n_qubits == 0:
+            return "", 0, ([], [])
 
-    if n_qubits == 1:
-        return "0", 0, (nodes, [])
+        if n_qubits == 1:
+            return "0", 0, (nodes, [])
 
-    if n_qubits == 2:
-        weight = G_m[0, 1]
-        if weight < 0.0:
-            return "00", 0, (nodes, [])
+        if n_qubits == 2:
+            weight = G_m[0, 1]
+            if weight < 0.0:
+                return "00", 0, (nodes, [])
 
-        return "01", weight, ([nodes[0]], [nodes[1]])
+            return "01", weight, ([nodes[0]], [nodes[1]])
 
     if quality is None:
         quality = 8
 
     if shots is None:
         # Number of measurement shots
-        shots = n_qubits << quality
+        shots = 1 << quality
 
     n_steps = 1 << quality
     grid_size = n_steps * n_qubits
